<?php

namespace Nwb\CyberSecModule;

use Nwb\CyberSecModule\Config\SecurityPatterns;

/**
 * Module de remontée de piratage
 *
 * Ce module détecte les requêtes malveillantes et les remonte
 * automatiquement à un endpoint centralisé.
 */
class CyberSecModule
{
    private $config;
    private $queueFile;
    private $patterns;

    /**
     * Configuration par défaut
     */
    private const DEFAULT_CONFIG = [
        'endpoint' => 'https://n8n.kub.nwb.fr/webhook/signalHackingAttempt',
        'timeout' => 2,
        'max_retries' => 3,
        'queue_file' => '/tmp/cybersec_queue.json',
        'compression' => true
    ];

    /**
     * Clés sensibles à hacher
     */
    private const SENSITIVE_KEYS = [
        'HTTP_AUTHORIZATION',
        'HTTP_COOKIE',
        'PHP_AUTH_PW',
        'PHP_AUTH_USER'
    ];

    public function __construct(array $config = [])
    {
        $this->config = $this->buildConfig($config);
        $this->queueFile = $this->config['queue_file'];
        $this->patterns = SecurityPatterns::getAllPatterns();
    }

    /**
     * Ajoute des patterns personnalisés à une catégorie existante
     *
     * @param string $category Catégorie (uri, post, headers)
     * @param array $patterns Nouveaux patterns à ajouter
     * @return void
     */
    public function addCustomPatterns(string $category, array $patterns): void
    {
        if (!isset($this->patterns[$category])) {
            throw new \InvalidArgumentException(
                "Catégorie invalide: $category. Catégories disponibles: " . implode(', ', array_keys($this->patterns))
            );
        }

        $this->patterns[$category] = array_merge($this->patterns[$category], $patterns);
    }

    /**
     * Récupère les patterns actuellement chargés
     *
     * @param string|null $category Catégorie spécifique ou null pour toutes
     * @return array
     */
    public function getPatterns(?string $category = null): array
    {
        if ($category === null) {
            return $this->patterns;
        }

        if (!isset($this->patterns[$category])) {
            throw new \InvalidArgumentException("Catégorie invalide: $category");
        }

        return $this->patterns[$category];
    }

    /**
     * Signale une tentative de piratage
     *
     * @param string $pattern Description du motif détecté
     * @param array $additionalData Données supplémentaires optionnelles
     * @return bool True si le signal a été envoyé avec succès
     */
    public function signalHackingAttempt(string $pattern, array $additionalData = []): bool
    {
        $payload = $this->buildPayload($pattern, $additionalData);

        if ($this->sendPayload($payload)) {
            return true;
        }

        // En cas d'échec, ajouter à la file d'attente
        $this->addToQueue($payload);
        return false;
    }

    /**
     * Détecte automatiquement les menaces et les signale
     *
     * @return bool True si une menace a été détectée et signalée
     */
    public function detectAndSignal(): bool
    {
        $detectedPattern = null;

        // Vérification URI
        if (isset($_SERVER['REQUEST_URI'])) {
            $detectedPattern = $this->checkPatterns($_SERVER['REQUEST_URI'], $this->patterns['uri']);
            if ($detectedPattern) {
                return $this->signalHackingAttempt("URI malveillante détectée: " . $detectedPattern);
            }
        }

        // Vérification POST
        if (!empty($_POST)) {
            $postData = json_encode($_POST);
            foreach ($this->patterns['post'] as $pattern) {
                // Échapper les caractères spéciaux pour les expressions régulières
                $escapedPattern = str_replace('/', '\/', $pattern);
                if (preg_match('/' . $escapedPattern . '/i', $postData)) {
                    return $this->signalHackingAttempt("Données POST malveillantes détectées: " . $pattern);
                }
            }
        }

        // Vérification headers
        foreach ($this->patterns['headers'] as $pattern) {
            $userAgent = $_SERVER['HTTP_USER_AGENT'] ?? '';
            if (stripos($userAgent, $pattern) !== false) {
                return $this->signalHackingAttempt("User-Agent malveillant détecté: " . $pattern);
            }
        }

        return false;
    }

    /**
     * Traite la file d'attente des signalements en échec
     *
     * @return int Nombre d'éléments traités avec succès
     */
    public function processQueue(): int
    {
        if (!file_exists($this->queueFile)) {
            return 0;
        }

        $content = file_get_contents($this->queueFile);
        if (empty($content)) {
            return 0;
        }

        $queue = json_decode($content, true);
        if (!is_array($queue)) {
            return 0;
        }

        $processed = 0;
        $remaining = [];

        foreach ($queue as $payload) {
            if ($this->sendPayload($payload)) {
                $processed++;
            } else {
                $remaining[] = $payload;
            }
        }

        // Mise à jour de la file d'attente
        if (empty($remaining)) {
            unlink($this->queueFile);
        } else {
            file_put_contents($this->queueFile, json_encode($remaining, JSON_PRETTY_PRINT));
        }

        return $processed;
    }

    /**
     * Construit la configuration finale en fusionnant les sources
     */
    private function buildConfig(array $config): array
    {
        $finalConfig = self::DEFAULT_CONFIG;

        // Variables d'environnement
        if ($envEndpoint = getenv('CYBERSECMODULE_URL')) {
            $finalConfig['endpoint'] = $envEndpoint;
        }

        // Configuration passée en paramètre (priorité maximale)
        $finalConfig = array_merge($finalConfig, $config);

        // Token d'authentification
        $finalConfig['auth_token'] = $config['auth_token'] ?? getenv('CYBERSECMODULE_TOKEN');

        return $finalConfig;
    }

    /**
     * Construit le payload JSON à envoyer
     */
    private function buildPayload(string $pattern, array $additionalData = []): array
    {
        $ip = $this->getClientIp();
        $timestamp = date('c'); // ISO 8601

        $data = [
            'get' => $_GET,
            'post' => $_POST,
            'server' => $_SERVER,
            'cookie' => $_COOKIE
        ];

        // Ajout des données supplémentaires
        if (!empty($additionalData)) {
            $data['additional'] = $additionalData;
        }

        // Sanitisation des données sensibles
        $data = $this->sanitizeData($data);

        return [
            'ip' => $ip,
            'timestamp' => $timestamp,
            'pattern' => $pattern,
            'data' => $data
        ];
    }

    /**
     * Sanitise les données en hachant les éléments sensibles
     */
    private function sanitizeData(array $data): array
    {
        foreach ($data as $section => &$sectionData) {
            if (is_array($sectionData)) {
                foreach ($sectionData as $key => &$value) {
                    if (in_array($key, self::SENSITIVE_KEYS) || $this->isSensitiveKey($key)) {
                        $value = hash('sha256', (string)$value);
                    }
                }
            }
        }

        return $data;
    }

    /**
     * Vérifie si une clé est sensible (cookies de session, etc.)
     */
    private function isSensitiveKey(string $key): bool
    {
        $sensitivePatterns = [
            '/^SESS_/',
            '/^PHPSESSID/',
            '/session/i',
            '/token/i',
            '/auth/i',
            '/password/i',
            '/passwd/i'
        ];

        foreach ($sensitivePatterns as $pattern) {
            if (preg_match($pattern, $key)) {
                return true;
            }
        }

        return false;
    }

    /**
     * Envoie le payload vers l'endpoint
     */
    private function sendPayload(array $payload): bool
    {
        $retries = 0;
        $maxRetries = $this->config['max_retries'];

        while ($retries < $maxRetries) {
            if ($this->performHttpRequest($payload)) {
                return true;
            }

            $retries++;
            if ($retries < $maxRetries) {
                // Backoff exponentiel: 1s, 2s, 4s
                sleep(pow(2, $retries - 1));
            }
        }

        return false;
    }

    /**
     * Effectue la requête HTTP
     */
    private function performHttpRequest(array $payload): bool
    {
        if (!function_exists('curl_init')) {
            error_log('CyberSecModule: cURL extension is required');
            return false;
        }

        $ch = curl_init();
        $jsonData = json_encode($payload);

        // Compression GZIP optionnelle
        if ($this->config['compression'] && function_exists('gzencode')) {
            $jsonData = gzencode($jsonData);
            $headers = [
                'Content-Type: application/json',
                'Content-Encoding: gzip',
                'Authorization: Bearer ' . $this->config['auth_token']
            ];
        } else {
            $headers = [
                'Content-Type: application/json',
                'Authorization: Bearer ' . $this->config['auth_token']
            ];
        }

        curl_setopt_array($ch, [
            CURLOPT_URL => $this->config['endpoint'],
            CURLOPT_POST => true,
            CURLOPT_POSTFIELDS => $jsonData,
            CURLOPT_HTTPHEADER => $headers,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_TIMEOUT => $this->config['timeout'],
            CURLOPT_FOLLOWLOCATION => false,
            CURLOPT_SSL_VERIFYPEER => true,
            CURLOPT_USERAGENT => 'CyberSecModule/1.0'
        ]);

        $response = curl_exec($ch);
        $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        $error = curl_error($ch);
        curl_close($ch);

        if ($response === false || !empty($error)) {
            error_log("CyberSecModule HTTP Error: " . $error);
            return false;
        }

        // Codes de succès acceptés: 201 Created ou 204 No Content
        return in_array($httpCode, [201, 204]);
    }

    /**
     * Ajoute un payload à la file d'attente locale
     */
    private function addToQueue(array $payload): void
    {
        $queue = [];

        if (file_exists($this->queueFile)) {
            $content = file_get_contents($this->queueFile);
            if (!empty($content)) {
                $queue = json_decode($content, true) ?: [];
            }
        }

        $queue[] = $payload;

        // Limitation de la taille de la file (max 100 éléments)
        if (count($queue) > 100) {
            $queue = array_slice($queue, -100);
        }

        file_put_contents($this->queueFile, json_encode($queue, JSON_PRETTY_PRINT));
    }

    /**
     * Vérifie les motifs dans une chaîne
     */
    private function checkPatterns(string $input, array $patterns): ?string
    {
        foreach ($patterns as $pattern) {
            if (stripos($input, $pattern) !== false) {
                return $pattern;
            }
        }

        return null;
    }

    /**
     * Récupère l'adresse IP du client
     */
    private function getClientIp(): string
    {
        $ipKeys = [
            'HTTP_CF_CONNECTING_IP',     // Cloudflare
            'HTTP_CLIENT_IP',
            'HTTP_X_FORWARDED_FOR',
            'HTTP_X_FORWARDED',
            'HTTP_FORWARDED_FOR',
            'HTTP_FORWARDED',
            'REMOTE_ADDR'
        ];

        foreach ($ipKeys as $key) {
            if (!empty($_SERVER[$key])) {
                $ip = $_SERVER[$key];

                // Si plusieurs IPs (proxy chain), prendre la première
                if (strpos($ip, ',') !== false) {
                    $ip = explode(',', $ip)[0];
                }

                $ip = trim($ip);

                // Validation IP
                if (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_PRIV_RANGE | FILTER_FLAG_NO_RES_RANGE)) {
                    return $ip;
                }
            }
        }

        return $_SERVER['REMOTE_ADDR'] ?? 'unknown';
    }
}
