<?php

namespace Nwb\CyberSecModule\Config;

/**
 * Configuration des patterns de détection de menaces
 *
 * Ce fichier centralise tous les motifs utilisés pour détecter
 * les tentatives de piratage et facilite leur maintenance.
 */
class SecurityPatterns
{
    /**
     * Patterns de détection pour les URI malveillantes
     *
     * @return array
     */
    public static function getUriPatterns(): array
    {
        return [
            // === PATTERNS WORDPRESS ===
            '/wp-login.php',
            '/wp-admin',
            '/wordpress',
            '/wp-',

            // === FICHIERS SENSIBLES ===
            '/.env',
            '/.git',
            '.php.bak',

            // === FICHIERS MALVEILLANTS SPÉCIFIQUES ===
            // Backdoors et shells web communs
            'alf.php',
            'file5.php',
            'Jcrop.php',
            'dropdown.php',
            'css.php',
            'autoload_classmap.php',
            'lv.php',

            // === RÉPERTOIRES SENSIBLES ===
            '/cgi-bin/',
            '/plugins/',
            '/elfinder/',
            '/phpmyadmin',
            '/admin',

            // === FONCTIONS DANGEREUSES ===
            'eval(',
            'base64_decode'
        ];
    }

    /**
     * Patterns de détection pour les données POST malveillantes
     *
     * @return array
     */
    public static function getPostPatterns(): array
    {
        return [
            // === INJECTION SQL ===
            'UNION\s+SELECT',
            'DROP\s+TABLE',
            'INSERT\s+INTO',
            'DELETE\s+FROM',
            'UPDATE\s+SET',

            // === XSS (Cross-Site Scripting) ===
            '<script',
            'javascript:',
            'onload=',
            'onerror=',
            'onclick=',
            'onmouseover=',

            // === EXECUTION DE CODE ===
            'eval\(',
            'base64_decode',
            'exec\(',
            'system\(',
            'shell_exec\(',
            'passthru\(',

            // === INCLUSION DE FICHIERS ===
            'include\(',
            'require\(',
            'file_get_contents\(',

            // === AUTRES PATTERNS DANGEREUX ===
            'php://input',
            'php://filter',
            '../',
            '..\\',
        ];
    }

    /**
     * Patterns de détection pour les User-Agent malveillants
     *
     * @return array
     */
    public static function getHeaderPatterns(): array
    {
        return [
            // === OUTILS DE SCAN AUTOMATISÉS ===
            'sqlmap',
            'nikto',
            'nessus',
            'openvas',
            'nmap',
            'masscan',

            // === BOTS MALVEILLANTS ===
            'evil-bot',
            'bad-bot',
            'malicious',

            // === OUTILS D'EXPLOITATION ===
            'metasploit',
            'burpsuite',
            'havij',
            'acunetix',

            // === CRAWLERS AGRESSIFS ===
            'python-requests',
            'curl/',
            'wget/',

            // === SIGNATURES SPÉCIFIQUES ===
            'scanner',
            'exploit',
            'payload'
        ];
    }

    /**
     * Récupère tous les patterns organisés par catégorie
     *
     * @return array
     */
    public static function getAllPatterns(): array
    {
        return [
            'uri' => self::getUriPatterns(),
            'post' => self::getPostPatterns(),
            'headers' => self::getHeaderPatterns()
        ];
    }

    /**
     * Ajoute des patterns personnalisés à une catégorie existante
     *
     * @param string $category Catégorie (uri, post, headers)
     * @param array $patterns Nouveaux patterns à ajouter
     * @return array Patterns fusionnés
     */
    public static function addCustomPatterns(string $category, array $patterns): array
    {
        $allPatterns = self::getAllPatterns();

        if (!isset($allPatterns[$category])) {
            throw new \InvalidArgumentException("Catégorie invalide: $category");
        }

        $allPatterns[$category] = array_merge($allPatterns[$category], $patterns);

        return $allPatterns;
    }

    /**
     * Charge les patterns depuis un fichier JSON externe (optionnel)
     *
     * @param string $filePath Chemin vers le fichier JSON
     * @return array|null Patterns chargés ou null si erreur
     */
    public static function loadFromFile(string $filePath): ?array
    {
        if (!file_exists($filePath)) {
            return null;
        }

        $content = file_get_contents($filePath);
        if ($content === false) {
            return null;
        }

        $patterns = json_decode($content, true);
        if (json_last_error() !== JSON_ERROR_NONE) {
            return null;
        }

        return $patterns;
    }

    /**
     * Sauvegarde les patterns dans un fichier JSON
     *
     * @param string $filePath Chemin de destination
     * @param array $patterns Patterns à sauvegarder
     * @return bool Succès de l'opération
     */
    public static function saveToFile(string $filePath, array $patterns): bool
    {
        $json = json_encode($patterns, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES);
        if ($json === false) {
            return false;
        }

        return file_put_contents($filePath, $json) !== false;
    }
}
